// 
// Better ADL.
// Copyright (C) 2015, NorthTrader.
//

#region Using declarations
using System;
using System.Diagnostics;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.ComponentModel;
using System.Xml.Serialization;
using NinjaTrader.Data;
using NinjaTrader.Gui.Chart;
#endregion

// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
    /// <summary>
	/// Better ADL: an adaption of Chaikin's Accumulation/Distribution Line, which attempts to quantify the amount of volume flowing into or out of an instrument.
    /// </summary>
    [Description("Better ADL: an adaption of Chaikin's Accumulation/Distribution Line, which attempts to quantify the amount of volume flowing into or out of an instrument.")]
    public class BADL : Indicator
    {
        #region Variables

		//Parameter defaults
		int iBarPeriod = 1;
		int iSpreadPeriod = 21;
		bool bIncludeGaps = false;

		//Other
		double dSpreadTotal, dSubTotal;
		bool bFirst, bSameAsChart;
		int iOldBarIndex;

		#endregion

        /// <summary>
        /// This method is used to configure the indicator and is called once before any bar data is loaded.
        /// </summary>
        protected override void Initialize()
        {
			CalculateOnBarClose = true;	 //Not updated on every tick
            Add(new Plot(Color.DarkGreen, PlotStyle.Line, "BAD"));
			if (iBarPeriod > 0) Add(PeriodType.Minute, iBarPeriod);  //Add sub data series if necessary
			bFirst = true;
        }

        /// <summary>
        /// Called on each bar update event (incoming tick) if CalculateOnBarClose = false. Otherwise called on bar close.
        /// </summary>
        protected override void OnBarUpdate()
        {
			if (bFirst) SetVars();

			if (BarsInProgress == 0)
			{
				//Main data series (bars shown on chart)
				if (bSameAsChart)
				{
					//Sub and Main are the same, so only process Main
					if (CurrentBar < iOldBarIndex)
					{
						//CurrentBar=0 for first bar of chart. Need iSpreadPeriod number of bars to begin calculating BADL.
						BAD.Set(0);
						if (CurrentBar < iSpreadPeriod) dSpreadTotal += dBarSpread();
						return;
					}
					else BAD.Set(BAD[1] + dBarValue());  //Update BADL
				}
				else
				{
					//Update BADL with sub data series total
					if (CurrentBar == 0) BAD.Set(dSubTotal);
					else BAD.Set(BAD[1] + dSubTotal);
					dSubTotal = 0;
				}
			}
			else if (!bSameAsChart)
			{
				//Sub data series (bars used for calculation if not same as Main)
				if (CurrentBar < iOldBarIndex)
				{
					//As above
					if (CurrentBar < iSpreadPeriod) dSpreadTotal += dBarSpread();
					return;
				}
				else dSubTotal += dBarValue();  //Update BADL
			}
				
        }

        #region Functions

		private double dBarValue()
		{
			//Return BADL value for current bar
			//Index [0] is current bar, [1] is previous bar

			//Update total spread
			double ldOldSpread, ldNewSpread;
			if (bIncludeGaps)
			{
				if (CurrentBar == iOldBarIndex) ldOldSpread = Close[iOldBarIndex] - Open[iOldBarIndex];
				else ldOldSpread = Close[iOldBarIndex] - Close[iOldBarIndex + 1];
				ldNewSpread = Close[1] - Close[2];
			}	
			else
			{
				ldOldSpread = Close[iOldBarIndex] - Open[iOldBarIndex];
				ldNewSpread = Close[1] - Open[1];
			}	
			ldOldSpread = Math.Abs(ldOldSpread);
			ldNewSpread = Math.Abs(ldNewSpread);
			dSpreadTotal = dSpreadTotal - ldOldSpread + ldNewSpread;

			//Calculate mean spread
			double ldSpreadMean = dSpreadTotal / iSpreadPeriod;
			if (ldSpreadMean == 0) ldSpreadMean = TickSize;  //can't divide by zero
			
			//Calculate and return BADL.
			double ldSpreadRatio = Math.Abs(Close[0] - Open[0]) / ldSpreadMean;
			double ldOpen = bIncludeGaps ? Close[1] : Open[0];
			double ldHigh = Math.Max(ldOpen, High[0]);
			double ldLow = Math.Min(ldOpen, Low[0]);
			
			return ldHigh == ldLow ? 0 : ((Close[0] - ldOpen) / (ldHigh - ldLow)) * Volume[0] * ldSpreadRatio;
		}

		private double dBarSpread()
		{
			//Return Open-Close spread of current bar
			//Allow for gaps if required
			double ldSpread = (!bIncludeGaps || CurrentBar == 0 ? Close[0] - Open[0] : Close[0] - Close[1]);
			return Math.Abs(ldSpread);
		}

		private string sDateTime(DateTime atD)
		{
			//Return DateTime as a standard formatted string
			return atD.ToString("dd-MMM-yyyy HH:mm");
		}

		private void SetVars()
		{
			//First call (for either data series)
			//Accesses properties that are unavailable in Initialize()
			//Set vars
			if (iBarPeriod == 0 || (BarsPeriod.Id == PeriodType.Minute && BarsPeriod.Value == iBarPeriod)) bSameAsChart = true;
			else bSameAsChart = false;
			bFirst = false;
			dSpreadTotal = dSubTotal = 0;
			iOldBarIndex = iSpreadPeriod + 1;
			
			//Write details to Output window
			ClearOutputWindow();
			Print(Instrument.FullName + ", Period = " + BarsPeriod.Id.ToString().Substring(0, 1) + BarsPeriod.Value + ", Pip Value = " + TickSize.ToString() +
				", Run time = " + sDateTime(DateTime.Now) + " (local)");
			Print("Bar calculation period = " + (bSameAsChart ? " same as chart bars" : "M" + iBarPeriod));
			Print("Spread period = " + iSpreadPeriod + " bars");
			Print("Include gap spreads = " + bIncludeGaps);
		}

		#endregion
		
        #region Properties
        [Browsable(false)]	// this line prevents the data series from being displayed in the indicator properties dialog, do not remove
        [XmlIgnore()]		// this line ensures that the indicator can be saved/recovered as part of a chart template, do not remove
        public DataSeries BAD
        {
            get { return Values[0]; }
        }

		//Parameters 
		[Description("Period of each bar (in minutes) on which the calculation is performed. 0 = same as chart bars.")]
		[GridCategory("Parameters")]
		[Gui.Design.DisplayName("\tBar calculation period")]
		public int BarPeriod
		{
            get { return iBarPeriod; }
            set { iBarPeriod = Math.Max(0, value); }
        }

		[Description("Number of previous bars to use for the mean spread calculation. Minimum = 1.")]
		[GridCategory("Parameters")]
		[Gui.Design.DisplayName("\tSpread period")]
		public int SpreadPeriod
		{
            get { return iSpreadPeriod; }
            set { iSpreadPeriod = Math.Max(1, value); }
        }


		[Description("True = use previous bar's Close for current bar's Open (include gap spreads). False = always use current bar's Open (ignore gap spreads).")]
		[GridCategory("Parameters")]
		[Gui.Design.DisplayName("Include gap spreads")]
		public bool IncludeGaps
		{
            get { return bIncludeGaps; }
            set { bIncludeGaps = value; }
        }

		#endregion
    }
}

#region NinjaScript generated code. Neither change nor remove.
// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
    public partial class Indicator : IndicatorBase
    {
        private BADL[] cacheBADL = null;

        private static BADL checkBADL = new BADL();

        /// <summary>
        /// Better ADL: an adaption of Chaikin's Accumulation/Distribution Line, which attempts to quantify the amount of volume flowing into or out of an instrument.
        /// </summary>
        /// <returns></returns>
        public BADL BADL(int barPeriod, bool includeGaps, int spreadPeriod)
        {
            return BADL(Input, barPeriod, includeGaps, spreadPeriod);
        }

        /// <summary>
        /// Better ADL: an adaption of Chaikin's Accumulation/Distribution Line, which attempts to quantify the amount of volume flowing into or out of an instrument.
        /// </summary>
        /// <returns></returns>
        public BADL BADL(Data.IDataSeries input, int barPeriod, bool includeGaps, int spreadPeriod)
        {
            if (cacheBADL != null)
                for (int idx = 0; idx < cacheBADL.Length; idx++)
                    if (cacheBADL[idx].BarPeriod == barPeriod && cacheBADL[idx].IncludeGaps == includeGaps && cacheBADL[idx].SpreadPeriod == spreadPeriod && cacheBADL[idx].EqualsInput(input))
                        return cacheBADL[idx];

            lock (checkBADL)
            {
                checkBADL.BarPeriod = barPeriod;
                barPeriod = checkBADL.BarPeriod;
                checkBADL.IncludeGaps = includeGaps;
                includeGaps = checkBADL.IncludeGaps;
                checkBADL.SpreadPeriod = spreadPeriod;
                spreadPeriod = checkBADL.SpreadPeriod;

                if (cacheBADL != null)
                    for (int idx = 0; idx < cacheBADL.Length; idx++)
                        if (cacheBADL[idx].BarPeriod == barPeriod && cacheBADL[idx].IncludeGaps == includeGaps && cacheBADL[idx].SpreadPeriod == spreadPeriod && cacheBADL[idx].EqualsInput(input))
                            return cacheBADL[idx];

                BADL indicator = new BADL();
                indicator.BarsRequired = BarsRequired;
                indicator.CalculateOnBarClose = CalculateOnBarClose;
#if NT7
                indicator.ForceMaximumBarsLookBack256 = ForceMaximumBarsLookBack256;
                indicator.MaximumBarsLookBack = MaximumBarsLookBack;
#endif
                indicator.Input = input;
                indicator.BarPeriod = barPeriod;
                indicator.IncludeGaps = includeGaps;
                indicator.SpreadPeriod = spreadPeriod;
                Indicators.Add(indicator);
                indicator.SetUp();

                BADL[] tmp = new BADL[cacheBADL == null ? 1 : cacheBADL.Length + 1];
                if (cacheBADL != null)
                    cacheBADL.CopyTo(tmp, 0);
                tmp[tmp.Length - 1] = indicator;
                cacheBADL = tmp;
                return indicator;
            }
        }
    }
}

// This namespace holds all market analyzer column definitions and is required. Do not change it.
namespace NinjaTrader.MarketAnalyzer
{
    public partial class Column : ColumnBase
    {
        /// <summary>
        /// Better ADL: an adaption of Chaikin's Accumulation/Distribution Line, which attempts to quantify the amount of volume flowing into or out of an instrument.
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.BADL BADL(int barPeriod, bool includeGaps, int spreadPeriod)
        {
            return _indicator.BADL(Input, barPeriod, includeGaps, spreadPeriod);
        }

        /// <summary>
        /// Better ADL: an adaption of Chaikin's Accumulation/Distribution Line, which attempts to quantify the amount of volume flowing into or out of an instrument.
        /// </summary>
        /// <returns></returns>
        public Indicator.BADL BADL(Data.IDataSeries input, int barPeriod, bool includeGaps, int spreadPeriod)
        {
            return _indicator.BADL(input, barPeriod, includeGaps, spreadPeriod);
        }
    }
}

// This namespace holds all strategies and is required. Do not change it.
namespace NinjaTrader.Strategy
{
    public partial class Strategy : StrategyBase
    {
        /// <summary>
        /// Better ADL: an adaption of Chaikin's Accumulation/Distribution Line, which attempts to quantify the amount of volume flowing into or out of an instrument.
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.BADL BADL(int barPeriod, bool includeGaps, int spreadPeriod)
        {
            return _indicator.BADL(Input, barPeriod, includeGaps, spreadPeriod);
        }

        /// <summary>
        /// Better ADL: an adaption of Chaikin's Accumulation/Distribution Line, which attempts to quantify the amount of volume flowing into or out of an instrument.
        /// </summary>
        /// <returns></returns>
        public Indicator.BADL BADL(Data.IDataSeries input, int barPeriod, bool includeGaps, int spreadPeriod)
        {
            if (InInitialize && input == null)
                throw new ArgumentException("You only can access an indicator with the default input/bar series from within the 'Initialize()' method");

            return _indicator.BADL(input, barPeriod, includeGaps, spreadPeriod);
        }
    }
}
#endregion
